<?php

namespace App\Http\Controllers;

use App\Http\Requests\AdminLoginRequest;
use App\Http\Requests\AdminResetPasswordRequest;
use App\Jobs\SendForgetMail;
use App\Models\Admin;
use App\Models\Calibration;
use App\Models\CalibrationLabel;
use App\Models\Customer;
use App\Models\Instrument;
use App\Models\MasterEquipment;
use App\Models\PerformanceQualificationCertificate;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\Validator;

class AdminController extends Controller
{
    protected function login()
    {
        return view('auth.login');
    }

    protected function loginProcess(AdminLoginRequest $request)
    {
        $request = $request->validated();

        $attempt = (New Admin)->login($request['email'], $request['password']);
        
        if($attempt === 404) toastr('Email Not Found!', 'error', 'Error');
        elseif ($attempt) return redirect()->route('admin_index');
        else toastr('Invalid Credentials', 'warning', 'Login Failed');
        return back();
    }

    protected function forgetPwdProcess()
    {
        $email = env('ADMIN_EMAIL');
        $token = uniqid();
        (new Admin())->setResetToken($email, $token);
        dispatch_now(new SendForgetMail($email, $token));
        toastr('Mail Sent', 'success', 'Success');
        return back();
    }

    protected function resetPwd()
    {
        return view('auth.reset-password');
    }

    protected function resetPwdProcess(AdminResetPasswordRequest $request, $token)
    {
        $request = $request->validated();
        $admin = (new Admin())->resetPassword($token, $request['password']);
        if($admin)
        {
            toastr('Password Updated', 'success', 'Reset Password');
            return redirect()->route('admin_login');
        }
            
        toastr('Token Expired', 'warning', 'Token'); 
        return back();
    }

    protected function logout()
    {
        auth('admin')->logout();
        return back();
    }

    protected function profile()
    {
        return view('admin.page.profile');
    }

    protected function profileProcess()
    {
        $validator = Validator::make(request()->all(),
        [
            'fname' => 'required',
            'lname' => 'required',
            'email' => "required|unique:admins,email,".auth('admin')->id(),
        ]);

        if(!empty(request()->password))
        {
            $validator = Validator::make(request()->all(),
            [
                'fname' => 'required',
                'lname' => 'required',
                'email' => "required|unique:admins,email,".auth('admin')->id(),
                'password' => 'confirmed|min:6|max:60',
                'password_confirmation' => 'required|min:6|max:60',
            ]);
        }

        if(!empty(request()->password) && request()->hasFile('image'))
        {
            $validator = Validator::make(request()->all(),
            [
                'fname' => 'required',
                'lname' => 'required',
                'email' => "required|unique:admins,email,".auth('admin')->id(),
                'image' => 'mimes:jpeg,jpg,png,gif|max:30000',
                'password' => 'confirmed|min:6|max:60',
                'password_confirmation' => 'required|min:6|max:60',
            ]);
        }

        if (request()->hasFile('image'))
        {
            $validator = Validator::make(request()->all(),
            [
                'fname' => 'required',
                'lname' => 'required',
                'email' => "required|unique:admins,email,".auth('admin')->id(),
                'image' => 'mimes:jpeg,jpg,png,gif|max:30000',
            ]);
        }

        if ($validator->fails())
        {
            return back()->withErrors($validator)->withInput();
        }

        $admin = Admin::find(auth('admin')->id());
        $admin->fname = request()->fname;
        $admin->lname = request()->lname;
        $admin->email = request()->email;
        if (!empty(request()->password))
        {
            $admin->password = bcrypt(request()->password);
        }
        if (request()->hasFile('image'))
        {
            // dd(1);
            $admin->image = request()->image->getClientOriginalName();
            request()->image->move(public_path('uploads/admin/profile/pics/'), request()->image->getClientOriginalName());
        }
        if($admin->update())
        {
            toastr('Profile Updated', 'success', 'Success');
            return back();
        }
    }

    protected function index()
    {
        $data = [];
        $data['masterEquipmentCount'] = MasterEquipment::count();
        $data['calibrationCount'] = Calibration::where(['certificate_id'=>null, 'renewed'=>0])->count();
        $data['pqc'] = PerformanceQualificationCertificate::where('renewed',0)->count();
        $data['customerCount'] = Customer::count();
        $data['instrumentCount'] = Instrument::count();
        $data['labelCount'] = CalibrationLabel::count();
        return view('page.dashboard', compact('data'));
    }

    protected function command($command)
    {
        Artisan::call($command);
        return Artisan::output();
    }
}
